@@ -41,7 +41,8 @@ print.keras_training_history <- function(x, ...) {
 #' 
 #' Plots metrics recorded during training. 
 #' 
-#' @param x Training history object returned from `fit()`.
+#' @param x Training history object returned from 
+#'  `fit.keras.engine.training.Model()`.
 #' @param y Unused.
 #' @param metrics One or more metrics to plot (e.g. `c('loss', 'accuracy')`).
 #'   Defaults to plotting all captured metrics.
@@ -195,9 +196,12 @@ to_keras_training_history <- function(history) {
   # turn history into an R object so it can be persited and
   # and give it a class so we can write print/plot methods
   params <- history$params
-  if (params$do_validation)
-    params$validation_samples <- dim(history$validation_data[[1]])[[1]]
-  
+  if (params$do_validation) {
+    if (!is.null(params$validation_steps))
+      params$validation_samples <- params$validation_steps
+    else
+      params$validation_samples <- dim(history$validation_data[[1]])[[1]]
+  }
   # normalize metrics
   metrics <- history$history
   metrics <- lapply(metrics, function(metric) {
