@@ -101,11 +101,20 @@ layer_gaussian_dropout <- function(object, rate, input_shape = NULL,
 #' @family noise layers   
 #'
 #' @export
-layer_alpha_dropout <- function(object, rate, noise_shape = NULL, seed = NULL) {
+layer_alpha_dropout <- function(object, rate, noise_shape = NULL, seed = NULL, input_shape = NULL,
+                                batch_input_shape = NULL, batch_size = NULL, dtype = NULL, 
+                                name = NULL, trainable = NULL, weights = NULL) {
   create_layer(keras$layers$AlphaDropout, object, list(
     rate = rate,
     noise_shape = noise_shape,
-    seed = as_nullable_integer(seed)
+    seed = as_nullable_integer(seed),
+    input_shape = normalize_shape(input_shape),
+    batch_input_shape = normalize_shape(batch_input_shape),
+    batch_size = as_nullable_integer(batch_size),
+    dtype = dtype,
+    name = name,
+    trainable = trainable,
+    weights = weights
   ))
 }
 
