@@ -256,13 +256,14 @@ clone_model <- function(model, input_tensors = NULL) {
 #' @family model functions
 #'
 #' @export
-compile <- function(object, optimizer, loss, 
-                    metrics = NULL, 
-                    loss_weights = NULL,
-                    sample_weight_mode = NULL, 
-                    weighted_metrics = NULL,
-                    target_tensors = NULL,
-                    ...) {
+compile.keras.engine.training.Model <-
+  function(object, optimizer, loss, 
+           metrics = NULL, 
+           loss_weights = NULL,
+           sample_weight_mode = NULL, 
+           weighted_metrics = NULL,
+           target_tensors = NULL,
+           ...) {
   
   # give losses a name
   loss_name <- deparse(substitute(loss))
@@ -400,13 +401,14 @@ compile <- function(object, optimizer, loss,
 #' @family model functions
 #'
 #' @export
-fit <- function(object, x = NULL, y = NULL, batch_size=NULL, epochs=10, 
-                verbose=getOption("keras.fit_verbose", default = 1), callbacks=NULL,
-                view_metrics = getOption("keras.view_metrics", default = "auto"),
-                validation_split=0.0, validation_data=NULL, shuffle=TRUE,
-                class_weight=NULL, sample_weight=NULL, initial_epoch=0,
-                steps_per_epoch=NULL, validation_steps=NULL, ...) {
-  
+fit.keras.engine.training.Model <- 
+  function(object, x = NULL, y = NULL, batch_size=NULL, epochs=10, 
+           verbose=getOption("keras.fit_verbose", default = 1), callbacks=NULL,
+           view_metrics = getOption("keras.view_metrics", default = "auto"),
+           validation_split=0.0, validation_data=NULL, shuffle=TRUE,
+           class_weight=NULL, sample_weight=NULL, initial_epoch=0,
+           steps_per_epoch=NULL, validation_steps=NULL, ...) {
+    
   # defaults
   if (is.null(batch_size) && is.null(steps_per_epoch))
     batch_size <- 32L
@@ -469,7 +471,7 @@ fit <- function(object, x = NULL, y = NULL, batch_size=NULL, epochs=10,
 
 #' Evaluate a Keras model
 
-#' @inheritParams fit
+#' @inheritParams fit.keras.engine.training.Model
 #'
 #' @param object Model object to evaluate
 #' @param x Vector, matrix, or array of test data (or list if the model has
@@ -700,7 +702,7 @@ test_on_batch <- function(object, x, y, sample_weight = NULL) {
 #' this allows you to do real-time data augmentation on images on CPU in
 #' parallel to training your model on GPU.
 #' 
-#' @inheritParams fit 
+#' @inheritParams fit.keras.engine.training.Model 
 #'
 #' @param object Keras model object
 #' @param generator A generator (e.g. like the one provided by
@@ -932,6 +934,10 @@ as_generator.function <- function(x) {
   reticulate::py_iterator(function() keras_array(x()))
 }
 
+as_generator.keras_preprocessing.sequence.TimeseriesGenerator <- function(x) {
+  reticulate::as_iterator(x)
+}
+
 is_main_thread_generator <- function(x) {
   UseMethod("is_main_thread_generator")
 }
@@ -957,6 +963,15 @@ is_main_thread_generator.keras.preprocessing.image.Iterator <- function(x) {
   }
 }
 
+is_main_thread_generator.keras_preprocessing.image.Iterator <- function(x) {
+  if (py_has_attr(x, "image_data_generator")) {
+    generator <- x$image_data_generator
+    !is.null(generator$preprocessing_function)
+  } else {
+    FALSE
+  }
+}
+
 is_tensorflow_dataset <- function(x) {
   inherits(x, "tensorflow.python.data.ops.dataset_ops.Dataset")
 }
