@@ -67,6 +67,7 @@ import inspect
 import os
 import shutil
 
+import keras
 from keras import utils
 from keras import layers
 from keras.layers import advanced_activations
@@ -80,6 +81,7 @@ from keras import losses
 from keras import metrics
 from keras import backend
 from keras import activations
+from keras import preprocessing
 
 import sys
 if sys.version[0] == '2':
@@ -89,6 +91,7 @@ if sys.version[0] == '2':
 
 EXCLUDE = {
     'Optimizer',
+    'TFOptimizer',
     'Wrapper',
     'get_session',
     'set_session',
@@ -97,14 +100,23 @@ EXCLUDE = {
     'deserialize',
     'get',
     'set_image_dim_ordering',
+    'normalize_data_format',
     'image_dim_ordering',
     'get_variable_shape',
 }
 
+
+# For each class to document, it is possible to:
+# 1) Document only the class: [classA, classB, ...]
+# 2) Document all its methods: [classA, (classB, "*")]
+# 3) Choose which methods to document (methods listed as strings):
+# [classA, (classB, ["method1", "method2", ...]), ...]
+# 4) Choose which methods to document (methods listed as qualified names):
+# [classA, (classB, [module.classB.method1, module.classB.method2, ...]), ...]
 PAGES = [
     {
         'page': 'models/sequential.md',
-        'functions': [
+        'methods': [
             models.Sequential.compile,
             models.Sequential.fit,
             models.Sequential.evaluate,
@@ -120,7 +132,7 @@ PAGES = [
     },
     {
         'page': 'models/model.md',
-        'functions': [
+        'methods': [
             models.Model.compile,
             models.Model.fit,
             models.Model.evaluate,
@@ -141,12 +153,16 @@ PAGES = [
             layers.Activation,
             layers.Dropout,
             layers.Flatten,
+            layers.Input,
             layers.Reshape,
             layers.Permute,
             layers.RepeatVector,
             layers.Lambda,
             layers.ActivityRegularization,
             layers.Masking,
+            layers.SpatialDropout1D,
+            layers.SpatialDropout2D,
+            layers.SpatialDropout3D,
         ],
     },
     {
@@ -154,9 +170,11 @@ PAGES = [
         'classes': [
             layers.Conv1D,
             layers.Conv2D,
+            layers.SeparableConv1D,
             layers.SeparableConv2D,
             layers.Conv2DTranspose,
             layers.Conv3D,
+            layers.Conv3DTranspose,
             layers.Cropping1D,
             layers.Cropping2D,
             layers.Cropping3D,
@@ -181,6 +199,8 @@ PAGES = [
             layers.GlobalAveragePooling1D,
             layers.GlobalMaxPooling2D,
             layers.GlobalAveragePooling2D,
+            layers.GlobalMaxPooling3D,
+            layers.GlobalAveragePooling3D,
         ],
     },
     {
@@ -201,7 +221,6 @@ PAGES = [
             layers.SimpleRNNCell,
             layers.GRUCell,
             layers.LSTMCell,
-            layers.StackedRNNCells,
             layers.CuDNNGRU,
             layers.CuDNNLSTM,
         ],
@@ -247,6 +266,34 @@ PAGES = [
             layers.dot,
         ]
     },
+    {
+        'page': 'preprocessing/sequence.md',
+        'functions': [
+            preprocessing.sequence.pad_sequences,
+            preprocessing.sequence.skipgrams,
+            preprocessing.sequence.make_sampling_table,
+        ],
+        'classes': [
+            preprocessing.sequence.TimeseriesGenerator,
+        ]
+    },
+    {
+        'page': 'preprocessing/image.md',
+        'classes': [
+            (preprocessing.image.ImageDataGenerator, '*')
+        ]
+    },
+    {
+        'page': 'preprocessing/text.md',
+        'functions': [
+            preprocessing.text.hashing_trick,
+            preprocessing.text.one_hot,
+            preprocessing.text.text_to_word_sequence,
+        ],
+        'classes': [
+            preprocessing.text.Tokenizer,
+        ]
+    },
     {
         'page': 'layers/wrappers.md',
         'all_module_classes': [wrappers],
@@ -297,32 +344,6 @@ PAGES = [
 ROOT = 'http://keras.io/'
 
 
-def get_earliest_class_that_defined_member(member, cls):
-    ancestors = get_classes_ancestors([cls])
-    result = None
-    for ancestor in ancestors:
-        if member in dir(ancestor):
-            result = ancestor
-    if not result:
-        return cls
-    return result
-
-
-def get_classes_ancestors(classes):
-    ancestors = []
-    for cls in classes:
-        ancestors += cls.__bases__
-    filtered_ancestors = []
-    for ancestor in ancestors:
-        if ancestor.__name__ in ['object']:
-            continue
-        filtered_ancestors.append(ancestor)
-    if filtered_ancestors:
-        return filtered_ancestors + get_classes_ancestors(filtered_ancestors)
-    else:
-        return filtered_ancestors
-
-
 def get_function_signature(function, method=True):
     wrapped = getattr(function, '_original_function', None)
     if wrapped is None:
@@ -339,7 +360,7 @@ def get_function_signature(function, method=True):
         args = args[:-len(defaults)]
     else:
         kwargs = []
-    st = '%s.%s(' % (function.__module__, function.__name__)
+    st = '%s.%s(' % (clean_module_name(function.__module__), function.__name__)
 
     for a in args:
         st += str(a) + ', '
@@ -351,10 +372,6 @@ def get_function_signature(function, method=True):
         signature = st[:-2] + ')'
     else:
         signature = st + ')'
-
-    if not method:
-        # Prepend the module name.
-        signature = function.__module__ + '.' + signature
     return post_process_signature(signature)
 
 
@@ -365,12 +382,15 @@ def get_class_signature(cls):
     except (TypeError, AttributeError):
         # in case the class inherits from object and does not
         # define __init__
-        class_signature = cls.__module__ + '.' + cls.__name__ + '()'
+        class_signature = "{clean_module_name}.{cls_name}()".format(
+            clean_module_name=clean_module_name(cls.__module__),
+            cls_name=cls.__name__
+        )
     return post_process_signature(class_signature)
 
 
 def post_process_signature(signature):
-    parts = re.split('\.(?!\d)', signature)
+    parts = re.split(r'\.(?!\d)', signature)
     if len(parts) >= 4:
         if parts[1] == 'layers':
             signature = 'keras.layers.' + '.'.join(parts[3:])
@@ -381,21 +401,28 @@ def post_process_signature(signature):
     return signature
 
 
+def clean_module_name(name):
+    if name.startswith('keras_applications'):
+        name = name.replace('keras_applications', 'keras.applications')
+    if name.startswith('keras_preprocessing'):
+        name = name.replace('keras_preprocessing', 'keras.preprocessing')
+    assert name[:6] == 'keras.', 'Invalid module name: %s' % name
+    return name
+
+
 def class_to_docs_link(cls):
-    module_name = cls.__module__
-    assert module_name[:6] == 'keras.'
+    module_name = clean_module_name(cls.__module__)
     module_name = module_name[6:]
     link = ROOT + module_name.replace('.', '/') + '#' + cls.__name__.lower()
     return link
 
 
 def class_to_source_link(cls):
-    module_name = cls.__module__
-    assert module_name[:6] == 'keras.'
+    module_name = clean_module_name(cls.__module__)
     path = module_name.replace('.', '/')
     path += '.py'
     line = inspect.getsourcelines(cls)[-1]
-    link = ('https://github.com/fchollet/'
+    link = ('https://github.com/keras-team/'
             'keras/blob/master/' + path + '#L' + str(line))
     return '[[source]](' + link + ')'
 
@@ -408,13 +435,53 @@ def code_snippet(snippet):
 
 
 def count_leading_spaces(s):
-    ws = re.search('\S', s)
+    ws = re.search(r'\S', s)
     if ws:
         return ws.start()
     else:
         return 0
 
 
+def process_list_block(docstring, starting_point, leading_spaces, marker):
+    ending_point = docstring.find('\n\n', starting_point)
+    block = docstring[starting_point:(None if ending_point == -1 else
+                                      ending_point - 1)]
+    # Place marker for later reinjection.
+    docstring = docstring.replace(block, marker)
+    lines = block.split('\n')
+    # Remove the computed number of leading white spaces from each line.
+    lines = [re.sub('^' + ' ' * leading_spaces, '', line) for line in lines]
+    # Usually lines have at least 4 additional leading spaces.
+    # These have to be removed, but first the list roots have to be detected.
+    top_level_regex = r'^    ([^\s\\\(]+):(.*)'
+    top_level_replacement = r'- __\1__:\2'
+    lines = [re.sub(top_level_regex, top_level_replacement, line) for line in lines]
+    # All the other lines get simply the 4 leading space (if present) removed
+    lines = [re.sub(r'^    ', '', line) for line in lines]
+    # Fix text lines after lists
+    indent = 0
+    text_block = False
+    for i in range(len(lines)):
+        line = lines[i]
+        spaces = re.search(r'\S', line)
+        if spaces:
+            # If it is a list element
+            if line[spaces.start()] == '-':
+                indent = spaces.start() + 1
+                if text_block:
+                    text_block = False
+                    lines[i] = '\n' + line
+            elif spaces.start() < indent:
+                text_block = True
+                indent = spaces.start()
+                lines[i] = '\n' + line
+        else:
+            text_block = False
+            indent = 0
+    block = '\n'.join(lines)
+    return docstring, block
+
+
 def process_docstring(docstring):
     # First, extract code blocks and process them.
     code_blocks = []
@@ -454,19 +521,36 @@ def process_docstring(docstring):
             code_blocks.append(snippet)
             tmp = tmp[index:]
 
+    # Format docstring lists.
+    section_regex = r'\n( +)# (.*)\n'
+    section_idx = re.search(section_regex, docstring)
+    shift = 0
+    sections = {}
+    while section_idx and section_idx.group(2):
+        anchor = section_idx.group(2)
+        leading_spaces = len(section_idx.group(1))
+        shift += section_idx.end()
+        marker = '$' + anchor.replace(' ', '_') + '$'
+        docstring, content = process_list_block(docstring,
+                                                shift,
+                                                leading_spaces,
+                                                marker)
+        sections[marker] = content
+        section_idx = re.search(section_regex, docstring[shift:])
+
     # Format docstring section titles.
     docstring = re.sub(r'\n(\s+)# (.*)\n',
                        r'\n\1__\2__\n\n',
                        docstring)
-    # Format docstring lists.
-    docstring = re.sub(r'    ([^\s\\\(]+):(.*)\n',
-                       r'    - __\1__:\2\n',
-                       docstring)
 
-    # Strip all leading spaces.
+    # Strip all remaining leading spaces.
     lines = docstring.split('\n')
     docstring = '\n'.join([line.lstrip(' ') for line in lines])
 
+    # Reinject list blocks.
+    for marker, content in sections.items():
+        docstring = docstring.replace(marker, content)
+
     # Reinject code blocks.
     for i, code_block in enumerate(code_blocks):
         docstring = docstring.replace(
@@ -488,91 +572,126 @@ for subdir, dirs, fnames in os.walk('templates'):
             new_fpath = fpath.replace('templates', 'sources')
             shutil.copy(fpath, new_fpath)
 
-# Take care of index page.
-readme = open('../README.md').read()
-index = open('templates/index.md').read()
-index = index.replace('{{autogenerated}}', readme[readme.find('##'):])
-f = open('sources/index.md', 'w')
-f.write(index)
-f.close()
 
-print('Starting autogeneration.')
-for page_data in PAGES:
-    blocks = []
-    classes = page_data.get('classes', [])
-    for module in page_data.get('all_module_classes', []):
-        module_classes = []
+def read_file(path):
+    with open(path) as f:
+        return f.read()
+
+
+def collect_class_methods(cls, methods):
+    if isinstance(methods, (list, tuple)):
+        return [getattr(cls, m) if isinstance(m, str) else m for m in methods]
+    methods = []
+    for _, method in inspect.getmembers(cls, predicate=inspect.isroutine):
+        if method.__name__[0] == '_' or method.__name__ in EXCLUDE:
+            continue
+        methods.append(method)
+    return methods
+
+
+def render_function(function, method=True):
+    subblocks = []
+    signature = get_function_signature(function, method=method)
+    if method:
+        signature = signature.replace(
+            clean_module_name(function.__module__) + '.', '')
+    subblocks.append('### ' + function.__name__ + '\n')
+    subblocks.append(code_snippet(signature))
+    docstring = function.__doc__
+    if docstring:
+        subblocks.append(process_docstring(docstring))
+    return '\n\n'.join(subblocks)
+
+
+def read_page_data(page_data, type):
+    assert type in ['classes', 'functions', 'methods']
+    data = page_data.get(type, [])
+    for module in page_data.get('all_module_{}'.format(type), []):
+        module_data = []
         for name in dir(module):
             if name[0] == '_' or name in EXCLUDE:
                 continue
             module_member = getattr(module, name)
-            if inspect.isclass(module_member):
-                cls = module_member
-                if cls.__module__ == module.__name__:
-                    if cls not in module_classes:
-                        module_classes.append(cls)
-        module_classes.sort(key=lambda x: id(x))
-        classes += module_classes
+            if (inspect.isclass(module_member) and type == 'classes' or
+               inspect.isfunction(module_member) and type == 'functions'):
+                instance = module_member
+                if module.__name__ in instance.__module__:
+                    if instance not in module_data:
+                        module_data.append(instance)
+        module_data.sort(key=lambda x: id(x))
+        data += module_data
+    return data
 
-    for cls in classes:
-        subblocks = []
-        signature = get_class_signature(cls)
-        subblocks.append('<span style="float:right;">' +
-                         class_to_source_link(cls) + '</span>')
-        subblocks.append('### ' + cls.__name__ + '\n')
-        subblocks.append(code_snippet(signature))
-        docstring = cls.__doc__
-        if docstring:
-            subblocks.append(process_docstring(docstring))
-        blocks.append('\n'.join(subblocks))
 
-    functions = page_data.get('functions', [])
-    for module in page_data.get('all_module_functions', []):
-        module_functions = []
-        for name in dir(module):
-            if name[0] == '_' or name in EXCLUDE:
-                continue
-            module_member = getattr(module, name)
-            if inspect.isfunction(module_member):
-                function = module_member
-                if module.__name__ in function.__module__:
-                    if function not in module_functions:
-                        module_functions.append(function)
-        module_functions.sort(key=lambda x: id(x))
-        functions += module_functions
+if __name__ == '__main__':
+    readme = read_file('../README.md')
+    index = read_file('templates/index.md')
+    index = index.replace('{{autogenerated}}', readme[readme.find('##'):])
+    with open('sources/index.md', 'w') as f:
+        f.write(index)
 
-    for function in functions:
-        subblocks = []
-        signature = get_function_signature(function, method=False)
-        signature = signature.replace(function.__module__ + '.', '')
-        subblocks.append('### ' + function.__name__ + '\n')
-        subblocks.append(code_snippet(signature))
-        docstring = function.__doc__
-        if docstring:
-            subblocks.append(process_docstring(docstring))
-        blocks.append('\n\n'.join(subblocks))
+    print('Generating docs for Keras %s.' % keras.__version__)
+    for page_data in PAGES:
+        classes = read_page_data(page_data, 'classes')
 
-    if not blocks:
-        raise RuntimeError('Found no content for page ' +
-                           page_data['page'])
+        blocks = []
+        for element in classes:
+            if not isinstance(element, (list, tuple)):
+                element = (element, [])
+            cls = element[0]
+            subblocks = []
+            signature = get_class_signature(cls)
+            subblocks.append('<span style="float:right;">' +
+                             class_to_source_link(cls) + '</span>')
+            if element[1]:
+                subblocks.append('## ' + cls.__name__ + ' class\n')
+            else:
+                subblocks.append('### ' + cls.__name__ + '\n')
+            subblocks.append(code_snippet(signature))
+            docstring = cls.__doc__
+            if docstring:
+                subblocks.append(process_docstring(docstring))
+            methods = collect_class_methods(cls, element[1])
+            if methods:
+                subblocks.append('\n---')
+                subblocks.append('## ' + cls.__name__ + ' methods\n')
+                subblocks.append('\n---\n'.join(
+                    [render_function(method, method=True) for method in methods]))
+            blocks.append('\n'.join(subblocks))
 
-    mkdown = '\n----\n\n'.join(blocks)
-    # save module page.
-    # Either insert content into existing page,
-    # or create page otherwise
-    page_name = page_data['page']
-    path = os.path.join('sources', page_name)
-    if os.path.exists(path):
-        template = open(path).read()
-        assert '{{autogenerated}}' in template, ('Template found for ' + path +
-                                                 ' but missing {{autogenerated}} tag.')
-        mkdown = template.replace('{{autogenerated}}', mkdown)
-        print('...inserting autogenerated content into template:', path)
-    else:
-        print('...creating new page with autogenerated content:', path)
-    subdir = os.path.dirname(path)
-    if not os.path.exists(subdir):
-        os.makedirs(subdir)
-    open(path, 'w').write(mkdown)
+        methods = read_page_data(page_data, 'methods')
 
-shutil.copyfile('../CONTRIBUTING.md', 'sources/contributing.md')
+        for method in methods:
+            blocks.append(render_function(method, method=True))
+
+        functions = read_page_data(page_data, 'functions')
+
+        for function in functions:
+            blocks.append(render_function(function, method=False))
+
+        if not blocks:
+            raise RuntimeError('Found no content for page ' +
+                               page_data['page'])
+
+        mkdown = '\n----\n\n'.join(blocks)
+        # save module page.
+        # Either insert content into existing page,
+        # or create page otherwise
+        page_name = page_data['page']
+        path = os.path.join('sources', page_name)
+        if os.path.exists(path):
+            template = read_file(path)
+            assert '{{autogenerated}}' in template, ('Template found for ' + path +
+                                                     ' but missing {{autogenerated}}'
+                                                     ' tag.')
+            mkdown = template.replace('{{autogenerated}}', mkdown)
+            print('...inserting autogenerated content into template:', path)
+        else:
+            print('...creating new page with autogenerated content:', path)
+        subdir = os.path.dirname(path)
+        if not os.path.exists(subdir):
+            os.makedirs(subdir)
+        with open(path, 'w') as f:
+            f.write(mkdown)
+
+    shutil.copyfile('../CONTRIBUTING.md', 'sources/contributing.md')
