@@ -1,3 +1,9 @@
+"""Boston housing price regression dataset.
+"""
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import print_function
+
 from ..utils.data_utils import get_file
 import numpy as np
 
@@ -16,13 +22,13 @@ def load_data(path='boston_housing.npz', test_split=0.2, seed=113):
         Tuple of Numpy arrays: `(x_train, y_train), (x_test, y_test)`.
     """
     assert 0 <= test_split < 1
-    path = get_file(path,
-                    origin='https://s3.amazonaws.com/keras-datasets/boston_housing.npz',
-                    file_hash='f553886a1f8d56431e820c5b82552d9d95cfcb96d1e678153f8839538947dff5')
-    f = np.load(path)
-    x = f['x']
-    y = f['y']
-    f.close()
+    path = get_file(
+        path,
+        origin='https://s3.amazonaws.com/keras-datasets/boston_housing.npz',
+        file_hash='f553886a1f8d56431e820c5b82552d9d95cfcb96d1e678153f8839538947dff5')
+    with np.load(path) as f:
+        x = f['x']
+        y = f['y']
 
     np.random.seed(seed)
     indices = np.arange(len(x))
