@@ -1,5 +1,10 @@
 # -*- coding: utf-8 -*-
+"""Utilities common to CIFAR10 and CIFAR100 datasets.
+"""
 from __future__ import absolute_import
+from __future__ import division
+from __future__ import print_function
+
 import sys
 from six.moves import cPickle
 
@@ -15,17 +20,16 @@ def load_batch(fpath, label_key='labels'):
     # Returns
         A tuple `(data, labels)`.
     """
-    f = open(fpath, 'rb')
-    if sys.version_info < (3,):
-        d = cPickle.load(f)
-    else:
-        d = cPickle.load(f, encoding='bytes')
-        # decode utf8
-        d_decoded = {}
-        for k, v in d.items():
-            d_decoded[k.decode('utf8')] = v
-        d = d_decoded
-    f.close()
+    with open(fpath, 'rb') as f:
+        if sys.version_info < (3,):
+            d = cPickle.load(f)
+        else:
+            d = cPickle.load(f, encoding='bytes')
+            # decode utf8
+            d_decoded = {}
+            for k, v in d.items():
+                d_decoded[k.decode('utf8')] = v
+            d = d_decoded
     data = d['data']
     labels = d[label_key]
 
