@@ -1,3 +1,9 @@
+"""MNIST handwritten digits dataset.
+"""
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import print_function
+
 from ..utils.data_utils import get_file
 import numpy as np
 
