@@ -1,13 +1,17 @@
 # -*- coding: utf-8 -*-
+"""Locally-connected layers.
+"""
 from __future__ import absolute_import
+from __future__ import division
+from __future__ import print_function
 
 from .. import backend as K
 from .. import activations
 from .. import initializers
 from .. import regularizers
 from .. import constraints
-from ..engine import Layer
-from ..engine import InputSpec
+from ..engine.base_layer import Layer
+from ..engine.base_layer import InputSpec
 from ..utils import conv_utils
 from ..legacy import interfaces
 
@@ -34,7 +38,7 @@ class LocallyConnected1D(Layer):
 
     # Arguments
         filters: Integer, the dimensionality of the output space
-            (i.e. the number output of filters in the convolution).
+            (i.e. the number of output filters in the convolution).
         kernel_size: An integer or tuple/list of a single integer,
             specifying the length of the 1D convolution window.
         strides: An integer or tuple/list of a single integer,
@@ -97,7 +101,7 @@ class LocallyConnected1D(Layer):
         if self.padding != 'valid':
             raise ValueError('Invalid border mode for LocallyConnected1D '
                              '(only "valid" is supported): ' + padding)
-        self.data_format = conv_utils.normalize_data_format(data_format)
+        self.data_format = K.normalize_data_format(data_format)
         self.activation = activations.get(activation)
         self.use_bias = use_bias
         self.kernel_initializer = initializers.get(kernel_initializer)
@@ -147,8 +151,6 @@ class LocallyConnected1D(Layer):
         return (input_shape[0], length, self.filters)
 
     def call(self, inputs):
-        output_length, _, filters = self.kernel_shape
-
         output = K.local_conv1d(inputs, self.kernel, self.kernel_size, self.strides)
         if self.use_bias:
             output = K.bias_add(output, self.bias)
@@ -168,7 +170,8 @@ class LocallyConnected1D(Layer):
             'bias_initializer': initializers.serialize(self.bias_initializer),
             'kernel_regularizer': regularizers.serialize(self.kernel_regularizer),
             'bias_regularizer': regularizers.serialize(self.bias_regularizer),
-            'activity_regularizer': regularizers.serialize(self.activity_regularizer),
+            'activity_regularizer':
+                regularizers.serialize(self.activity_regularizer),
             'kernel_constraint': constraints.serialize(self.kernel_constraint),
             'bias_constraint': constraints.serialize(self.bias_constraint)
         }
@@ -186,12 +189,13 @@ class LocallyConnected2D(Layer):
 
     # Examples
     ```python
-        # apply a 3x3 unshared weights convolution with 64 output filters on a 32x32 image
-        # with `data_format="channels_last"`:
+        # apply a 3x3 unshared weights convolution with 64 output filters
+        # on a 32x32 image with `data_format="channels_last"`:
         model = Sequential()
         model.add(LocallyConnected2D(64, (3, 3), input_shape=(32, 32, 3)))
         # now model.output_shape == (None, 30, 30, 64)
-        # notice that this layer will consume (30*30)*(3*3*3*64) + (30*30)*64 parameters
+        # notice that this layer will consume (30*30)*(3*3*3*64)
+        # + (30*30)*64 parameters
 
         # add a 3x3 unshared weights convolution on top, with 32 output filters:
         model.add(LocallyConnected2D(32, (3, 3)))
@@ -200,7 +204,7 @@ class LocallyConnected2D(Layer):
 
     # Arguments
         filters: Integer, the dimensionality of the output space
-            (i.e. the number output of filters in the convolution).
+            (i.e. the number of output filters in the convolution).
         kernel_size: An integer or tuple/list of 2 integers, specifying the
             width and height of the 2D convolution window.
             Can be a single integer to specify the same value for
@@ -281,7 +285,7 @@ class LocallyConnected2D(Layer):
         if self.padding != 'valid':
             raise ValueError('Invalid border mode for LocallyConnected2D '
                              '(only "valid" is supported): ' + padding)
-        self.data_format = conv_utils.normalize_data_format(data_format)
+        self.data_format = K.normalize_data_format(data_format)
         self.activation = activations.get(activation)
         self.use_bias = use_bias
         self.kernel_initializer = initializers.get(kernel_initializer)
@@ -311,9 +315,10 @@ class LocallyConnected2D(Layer):
                                                    self.padding, self.strides[1])
         self.output_row = output_row
         self.output_col = output_col
-        self.kernel_shape = (output_row * output_col,
-                             self.kernel_size[0] * self.kernel_size[1] * input_filter,
-                             self.filters)
+        self.kernel_shape = (
+            output_row * output_col,
+            self.kernel_size[0] * self.kernel_size[1] * input_filter,
+            self.filters)
         self.kernel = self.add_weight(shape=self.kernel_shape,
                                       initializer=self.kernel_initializer,
                                       name='kernel',
@@ -352,8 +357,6 @@ class LocallyConnected2D(Layer):
             return (input_shape[0], rows, cols, self.filters)
 
     def call(self, inputs):
-        _, _, filters = self.kernel_shape
-
         output = K.local_conv2d(inputs,
                                 self.kernel,
                                 self.kernel_size,
@@ -362,8 +365,7 @@ class LocallyConnected2D(Layer):
                                 self.data_format)
 
         if self.use_bias:
-            if self.data_format == 'channels_first' or self.data_format == 'channels_last':
-                output = K.bias_add(output, self.bias, data_format=self.data_format)
+            output = K.bias_add(output, self.bias, data_format=self.data_format)
 
         output = self.activation(output)
         return output
@@ -381,7 +383,8 @@ class LocallyConnected2D(Layer):
             'bias_initializer': initializers.serialize(self.bias_initializer),
             'kernel_regularizer': regularizers.serialize(self.kernel_regularizer),
             'bias_regularizer': regularizers.serialize(self.bias_regularizer),
-            'activity_regularizer': regularizers.serialize(self.activity_regularizer),
+            'activity_regularizer':
+                regularizers.serialize(self.activity_regularizer),
             'kernel_constraint': constraints.serialize(self.kernel_constraint),
             'bias_constraint': constraints.serialize(self.bias_constraint)
         }
