@@ -1,4 +1,10 @@
-from ..engine.topology import Layer
+"""Layers that can merge several inputs into one.
+"""
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import print_function
+
+from ..engine.base_layer import Layer
 from .. import backend as K
 
 
@@ -38,7 +44,7 @@ class _Merge(Layer):
             return None
         elif len(shape1) < len(shape2):
             return self._compute_elemwise_op_output_shape(shape2, shape1)
-        elif len(shape2) == 0:
+        elif not shape2:
             return shape1
         output_shape = list(shape1[:-len(shape2)])
         for i, j in zip(shape1[-len(shape2):], shape2):
@@ -81,7 +87,8 @@ class _Merge(Layer):
                 shape = None
             else:
                 shape = input_shape[i][1:]
-            output_shape = self._compute_elemwise_op_output_shape(output_shape, shape)
+            output_shape = self._compute_elemwise_op_output_shape(output_shape,
+                                                                  shape)
         # If the inputs have different ranks, we have to reshape them
         # to make them broadcastable.
         if None not in input_shape and len(set(map(len, input_shape))) == 1:
@@ -90,6 +97,9 @@ class _Merge(Layer):
             self._reshape_required = True
 
     def call(self, inputs):
+        if not isinstance(inputs, list):
+            raise ValueError('A merge layer should be called '
+                             'on a list of inputs.')
         if self._reshape_required:
             reshaped_inputs = []
             input_ndims = list(map(K.ndim, inputs))
@@ -113,8 +123,10 @@ class _Merge(Layer):
                     if x_ndim is None:
                         x_shape = K.shape(x)
                         batch_size = x_shape[0]
-                        new_shape = K.concatenate([x_shape[1:], K.expand_dims(batch_size)])
-                        x_transposed = K.reshape(x, K.stack([batch_size, K.prod(x_shape[1:])]))
+                        new_shape = K.concatenate([x_shape[1:],
+                                                   K.expand_dims(batch_size)])
+                        x_transposed = K.reshape(x, K.stack([batch_size,
+                                                             K.prod(x_shape[1:])]))
                         x_transposed = K.permute_dimensions(x_transposed, (1, 0))
                         x_transposed = K.reshape(x_transposed, new_shape)
                         reshaped_inputs.append(x_transposed)
@@ -124,17 +136,20 @@ class _Merge(Layer):
                         reshaped_inputs.append(K.permute_dimensions(x, dims))
                         transposed = True
                     else:
-                        # We don't transpose inputs if they are 1D vectors or scalars.
+                        # We don't transpose inputs if they are
+                        # 1D vectors or scalars.
                         reshaped_inputs.append(x)
                 y = self._merge_function(reshaped_inputs)
                 y_ndim = K.ndim(y)
                 if transposed:
-                    # If inputs have been transposed, we have to transpose the output too.
+                    # If inputs have been transposed,
+                    # we have to transpose the output too.
                     if y_ndim is None:
                         y_shape = K.shape(y)
                         y_ndim = K.shape(y_shape)[0]
                         batch_size = y_shape[y_ndim - 1]
-                        new_shape = K.concatenate([K.expand_dims(batch_size), y_shape[:y_ndim - 1]])
+                        new_shape = K.concatenate([K.expand_dims(batch_size),
+                                                   y_shape[:y_ndim - 1]])
                         y = K.reshape(y, (-1, batch_size))
                         y = K.permute_dimensions(y, (1, 0))
                         y = K.reshape(y, new_shape)
@@ -155,7 +170,8 @@ class _Merge(Layer):
                 shape = None
             else:
                 shape = input_shape[i][1:]
-            output_shape = self._compute_elemwise_op_output_shape(output_shape, shape)
+            output_shape = self._compute_elemwise_op_output_shape(output_shape,
+                                                                  shape)
         batch_sizes = [s[0] for s in input_shape if s is not None]
         batch_sizes = set(batch_sizes)
         batch_sizes -= set([None])
@@ -197,7 +213,8 @@ class Add(_Merge):
         x1 = keras.layers.Dense(8, activation='relu')(input1)
         input2 = keras.layers.Input(shape=(32,))
         x2 = keras.layers.Dense(8, activation='relu')(input2)
-        added = keras.layers.Add()([x1, x2])  # equivalent to added = keras.layers.add([x1, x2])
+        # equivalent to added = keras.layers.add([x1, x2])
+        added = keras.layers.Add()([x1, x2])
 
         out = keras.layers.Dense(4)(added)
         model = keras.models.Model(inputs=[input1, input2], outputs=out)
@@ -235,13 +252,16 @@ class Subtract(_Merge):
     ```
     """
 
+    def build(self, input_shape):
+        super(Subtract, self).build(input_shape)
+        if len(input_shape) != 2:
+            raise ValueError('A `Subtract` layer should be called '
+                             'on exactly 2 inputs')
+
     def _merge_function(self, inputs):
         if len(inputs) != 2:
-            raise ValueError('`Subtract` layer should be called '
+            raise ValueError('A `Subtract` layer should be called '
                              'on exactly 2 inputs')
-        if inputs[0]._keras_shape != inputs[1]._keras_shape:
-            raise ValueError('`Subtract` layer should be called '
-                             'on inputs of the same shape')
         return inputs[0] - inputs[1]
 
 
@@ -321,12 +341,13 @@ class Concatenate(_Merge):
         super(Concatenate, self).__init__(**kwargs)
         self.axis = axis
         self.supports_masking = True
+        self._reshape_required = False
 
     def build(self, input_shape):
         # Used purely for shape validation.
-        if not isinstance(input_shape, list):
-            raise ValueError('`Concatenate` layer should be called '
-                             'on a list of inputs')
+        if not isinstance(input_shape, list) or len(input_shape) < 2:
+            raise ValueError('A `Concatenate` layer should be called '
+                             'on a list of at least 2 inputs')
         if all([shape is None for shape in input_shape]):
             return
         reduced_inputs_shapes = [list(shape) for shape in input_shape]
@@ -335,15 +356,12 @@ class Concatenate(_Merge):
             del reduced_inputs_shapes[i][self.axis]
             shape_set.add(tuple(reduced_inputs_shapes[i]))
         if len(shape_set) > 1:
-            raise ValueError('`Concatenate` layer requires '
+            raise ValueError('A `Concatenate` layer requires '
                              'inputs with matching shapes '
                              'except for the concat axis. '
                              'Got inputs shapes: %s' % (input_shape))
 
-    def call(self, inputs):
-        if not isinstance(inputs, list):
-            raise ValueError('A `Concatenate` layer should be called '
-                             'on a list of inputs.')
+    def _merge_function(self, inputs):
         return K.concatenate(inputs, axis=self.axis)
 
     def compute_output_shape(self, input_shape):
@@ -398,7 +416,7 @@ class Concatenate(_Merge):
 class Dot(_Merge):
     """Layer that computes a dot product between samples in two tensors.
 
-    E.g. if applied to two tensors `a` and `b` of shape `(batch_size, n)`,
+    E.g. if applied to a list of two tensors `a` and `b` of shape `(batch_size, n)`,
     the output will be a tensor of shape `(batch_size, 1)`
     where each entry `i` will be the dot product between
     `a[i]` and `b[i]`.
@@ -428,6 +446,7 @@ class Dot(_Merge):
         self.axes = axes
         self.normalize = normalize
         self.supports_masking = True
+        self._reshape_required = False
 
     def build(self, input_shape):
         # Used purely for shape validation.
@@ -451,7 +470,10 @@ class Dot(_Merge):
                 '%s != %s. ' % (shape1[axes[0]], shape2[axes[1]]) +
                 'Layer shapes: %s, %s' % (shape1, shape2))
 
-    def call(self, inputs):
+    def _merge_function(self, inputs):
+        if len(inputs) != 2:
+            raise ValueError('A `Dot` layer should be called '
+                             'on exactly 2 inputs')
         x1 = inputs[0]
         x2 = inputs[1]
         if isinstance(self.axes, int):
