@@ -4,14 +4,19 @@ import numpy as np
 
 from keras.layers import Dense, Dropout, Conv2D, InputLayer
 from keras import layers
-from keras.engine import Input, Layer, topology, get_source_inputs
+from keras.engine import Input, Layer, saving, get_source_inputs
 from keras.models import Model, Sequential
 from keras import backend as K
 from keras.models import model_from_json, model_from_yaml
-from keras.utils.test_utils import keras_test
+from keras.initializers import Constant
+
+
+skipif_no_tf_gpu = pytest.mark.skipif(
+    (K.backend() != 'tensorflow' or
+     not K.tensorflow_backend._get_available_gpus()),
+    reason='Requires TensorFlow backend and a GPU')
 
 
-@keras_test
 def test_get_updates_for():
     a = Input(shape=(2,))
     dense_layer = Dense(1)
@@ -22,7 +27,6 @@ def test_get_updates_for():
     assert dense_layer.get_updates_for(None) == [1]
 
 
-@keras_test
 def test_get_losses_for():
     a = Input(shape=(2,))
     dense_layer = Dense(1)
@@ -33,7 +37,6 @@ def test_get_losses_for():
     assert dense_layer.get_losses_for(None) == [1]
 
 
-@keras_test
 def test_trainable_weights():
     a = Input(shape=(2,))
     b = Dense(1)(a)
@@ -146,7 +149,6 @@ def test_learning_phase():
     assert fn_outputs_no_dp[1].sum() != fn_outputs_dp[1].sum()
 
 
-@keras_test
 def test_layer_call_arguments():
     # Test the ability to pass and serialize arguments to `call`.
     inp = layers.Input(shape=(2,))
@@ -166,7 +168,6 @@ def test_layer_call_arguments():
     assert not model.uses_learning_phase
 
 
-@keras_test
 def test_node_construction():
     ####################################################
     # test basics
@@ -177,9 +178,9 @@ def test_node_construction():
     assert a._keras_shape == (None, 32)
     a_layer, a_node_index, a_tensor_index = a._keras_history
     b_layer, b_node_index, b_tensor_index = b._keras_history
-    assert len(a_layer.inbound_nodes) == 1
+    assert len(a_layer._inbound_nodes) == 1
     assert a_tensor_index is 0
-    node = a_layer.inbound_nodes[a_node_index]
+    node = a_layer._inbound_nodes[a_node_index]
     assert node.outbound_layer == a_layer
 
     assert isinstance(node.inbound_layers, list)
@@ -202,18 +203,18 @@ def test_node_construction():
     a_2 = dense(a)
     b_2 = dense(b)
 
-    assert len(dense.inbound_nodes) == 2
-    assert len(dense.outbound_nodes) == 0
-    assert dense.inbound_nodes[0].inbound_layers == [a_layer]
-    assert dense.inbound_nodes[0].outbound_layer == dense
-    assert dense.inbound_nodes[1].inbound_layers == [b_layer]
-    assert dense.inbound_nodes[1].outbound_layer == dense
+    assert len(dense._inbound_nodes) == 2
+    assert len(dense._outbound_nodes) == 0
+    assert dense._inbound_nodes[0].inbound_layers == [a_layer]
+    assert dense._inbound_nodes[0].outbound_layer == dense
+    assert dense._inbound_nodes[1].inbound_layers == [b_layer]
+    assert dense._inbound_nodes[1].outbound_layer == dense
 
-    assert dense.inbound_nodes[0].input_tensors == [a]
-    assert dense.inbound_nodes[1].input_tensors == [b]
+    assert dense._inbound_nodes[0].input_tensors == [a]
+    assert dense._inbound_nodes[1].input_tensors == [b]
 
-    assert dense.inbound_nodes[0].get_config()['inbound_layers'] == ['input_a']
-    assert dense.inbound_nodes[1].get_config()['inbound_layers'] == ['input_b']
+    assert dense._inbound_nodes[0].get_config()['inbound_layers'] == ['input_a']
+    assert dense._inbound_nodes[1].get_config()['inbound_layers'] == ['input_b']
 
     # test layer properties
     test_layer = Dense(16, name='test_layer')
@@ -249,7 +250,6 @@ def test_node_construction():
     assert dense.get_output_mask_at(1) is None
 
 
-@keras_test
 def test_multi_input_layer():
     ####################################################
     # test multi-input layer
@@ -267,39 +267,28 @@ def test_multi_input_layer():
     assert merge_node_index == 0
     assert merge_tensor_index == 0
 
-    assert len(merge_layer.inbound_nodes) == 1
-    assert len(merge_layer.outbound_nodes) == 0
+    assert len(merge_layer._inbound_nodes) == 1
+    assert len(merge_layer._outbound_nodes) == 0
 
-    assert len(merge_layer.inbound_nodes[0].input_tensors) == 2
-    assert len(merge_layer.inbound_nodes[0].inbound_layers) == 2
+    assert len(merge_layer._inbound_nodes[0].input_tensors) == 2
+    assert len(merge_layer._inbound_nodes[0].inbound_layers) == 2
 
     c = Dense(64, name='dense_2')(merged)
     d = Dense(5, name='dense_3')(c)
 
     model = Model(inputs=[a, b], outputs=[c, d], name='model')
     assert len(model.layers) == 6
-    print('model.input_layers:', model.input_layers)
-    print('model.input_layers_node_indices:', model.input_layers_node_indices)
-    print('model.input_layers_tensor_indices:', model.input_layers_tensor_indices)
-    print('model.output_layers', model.output_layers)
-
-    print('output_shape:', model.compute_output_shape([(None, 32), (None, 32)]))
-    assert model.compute_output_shape([(None, 32), (None, 32)]) == [(None, 64), (None, 5)]
-
-    print('mask:', model.compute_mask([a, b], [None, None]))
+    expected_shapes = [(None, 64), (None, 5)]
+    assert model.compute_output_shape([(None, 32), (None, 32)]) == expected_shapes
     assert model.compute_mask([a, b], [None, None]) == [None, None]
-
-    print('output_shape:', model.compute_output_shape([(None, 32), (None, 32)]))
-    assert model.compute_output_shape([(None, 32), (None, 32)]) == [(None, 64), (None, 5)]
+    assert model.compute_output_shape([(None, 32), (None, 32)]) == expected_shapes
 
     # we don't check names of first 2 layers (inputs) because
     # ordering of same-level layers is not fixed
-    print('layers:', [layer.name for layer in model.layers])
-    assert [l.name for l in model.layers][2:] == ['dense_1', 'merge', 'dense_2', 'dense_3']
-    print('input_layers:', [l.name for l in model.input_layers])
-    assert [l.name for l in model.input_layers] == ['input_a', 'input_b']
-    print('output_layers:', [l.name for l in model.output_layers])
-    assert [l.name for l in model.output_layers] == ['dense_2', 'dense_3']
+    expected_names = ['dense_1', 'merge', 'dense_2', 'dense_3']
+    assert [l.name for l in model.layers][2:] == expected_names
+    assert [l.name for l in model._input_layers] == ['input_a', 'input_b']
+    assert [l.name for l in model._output_layers] == ['dense_2', 'dense_3']
 
     # actually run model
     fn = K.function(model.inputs, model.outputs)
@@ -309,7 +298,6 @@ def test_multi_input_layer():
     assert [x.shape for x in fn_outputs] == [(10, 64), (10, 5)]
 
     # test get_source_inputs
-    print(get_source_inputs(c))
     assert get_source_inputs(c) == [a, b]
 
     # serialization / deserialization
@@ -317,13 +305,9 @@ def test_multi_input_layer():
     recreated_model = model_from_json(json_config)
     recreated_model.compile('rmsprop', 'mse')
 
-    print('recreated:')
-    print([layer.name for layer in recreated_model.layers])
-    print([layer.name for layer in recreated_model.input_layers])
-    print([layer.name for layer in recreated_model.output_layers])
-    assert [l.name for l in recreated_model.layers][2:] == ['dense_1', 'merge', 'dense_2', 'dense_3']
-    assert [l.name for l in recreated_model.input_layers] == ['input_a', 'input_b']
-    assert [l.name for l in recreated_model.output_layers] == ['dense_2', 'dense_3']
+    assert [l.name for l in recreated_model.layers][2:] == expected_names
+    assert [l.name for l in recreated_model._input_layers] == ['input_a', 'input_b']
+    assert [l.name for l in recreated_model._output_layers] == ['dense_2', 'dense_3']
 
     fn = K.function(recreated_model.inputs, recreated_model.outputs)
     input_a_np = np.random.random((10, 32))
@@ -332,7 +316,6 @@ def test_multi_input_layer():
     assert [x.shape for x in fn_outputs] == [(10, 64), (10, 5)]
 
 
-@keras_test
 def test_recursion():
     ####################################################
     # test recursion
@@ -368,14 +351,10 @@ def test_recursion():
 
     # we don't check names of first 2 layers (inputs) because
     # ordering of same-level layers is not fixed
-    print('final_model layers:', [layer.name for layer in final_model.layers])
+    expected_shapes = [(10, 7), (10, 64)]
     assert [layer.name for layer in final_model.layers][2:] == ['model', 'dense_4']
-
-    print(model.compute_mask([e, f], [None, None]))
     assert model.compute_mask([e, f], [None, None]) == [None, None]
-
-    print(final_model.compute_output_shape([(10, 32), (10, 32)]))
-    assert final_model.compute_output_shape([(10, 32), (10, 32)]) == [(10, 7), (10, 64)]
+    assert final_model.compute_output_shape([(10, 32), (10, 32)]) == expected_shapes
 
     # run recursive model
     fn = K.function(final_model.inputs, final_model.outputs)
@@ -429,12 +408,7 @@ def test_recursion():
     assert [x.shape for x in fn_outputs] == [(10, 69)]
 
     # test serialization
-    print('multi_io_model.layers:', multi_io_model.layers)
-    print('len(model.inbound_nodes):', len(model.inbound_nodes))
-    print('len(model.outbound_nodes):', len(model.outbound_nodes))
     model_config = multi_io_model.get_config()
-    print(model_config)
-    print(json.dumps(model_config, indent=4))
     recreated_model = Model.from_config(model_config)
 
     fn = K.function(recreated_model.inputs, recreated_model.outputs)
@@ -462,14 +436,14 @@ def test_recursion():
     k = Input(shape=(32,), name='input_k')
     m, n = model([j, k])
 
-    with pytest.raises(TypeError):
+    with pytest.raises(ValueError):
         Model([j, k], [m, n])
 
     # disconnected graph
     j = Input(shape=(32,), name='input_j')
     k = Input(shape=(32,), name='input_k')
     m, n = model([j, k])
-    with pytest.raises(RuntimeError):
+    with pytest.raises(ValueError):
         Model([j], [m, n])
 
     # redundant outputs
@@ -490,7 +464,7 @@ def test_recursion():
     j = Input(shape=(32,), name='input_j')
     k = Input(shape=(32,), name='input_k')
     m, n = model([j, k])
-    with pytest.raises(TypeError):
+    with pytest.raises(ValueError):
         Model([j, k], [m, n, 0])
 
     ####################################################
@@ -521,9 +495,9 @@ def test_recursion():
         Dense(2)(x)
 
 
-@keras_test
 def test_load_layers():
-    from keras.layers import ConvLSTM2D, TimeDistributed, Bidirectional, Conv2D, Input
+    from keras.layers import ConvLSTM2D, TimeDistributed
+    from keras.layers import Bidirectional, Conv2D, Input
     from keras.models import Model
 
     if K.backend() == 'tensorflow' or K.backend() == 'cntk':
@@ -531,13 +505,14 @@ def test_load_layers():
     else:
         inputs = Input(shape=(10, 1, 20, 20))
     td_conv = TimeDistributed(Conv2D(15, (5, 5)))(inputs)
-    bi_convlstm2d = Bidirectional(ConvLSTM2D(10, (3, 3)), merge_mode='concat')(td_conv)
-    model = Model(inputs=inputs, outputs=bi_convlstm2d)
+    bi_conv = Bidirectional(ConvLSTM2D(10, (3, 3)), merge_mode='concat')(td_conv)
+    model = Model(inputs=inputs, outputs=bi_conv)
 
     weight_value_tuples = []
 
     # TimeDistributed Conv2D layer
-    # use 'channels_first' data format to check that the function is being called correctly for Conv2D
+    # use 'channels_first' data format to check that
+    # the function is being called correctly for Conv2D
     # old: (filters, stack_size, kernel_rows, kernel_cols)
     # new: (kernel_rows, kernel_cols, stack_size, filters)
     weight_tensor_td_conv_old = list()
@@ -545,7 +520,7 @@ def test_load_layers():
     weight_tensor_td_conv_old.append(np.zeros((15,)))
     td_conv_layer = model.layers[1]
     td_conv_layer.layer.data_format = 'channels_first'
-    weight_tensor_td_conv_new = topology.preprocess_weights_for_loading(
+    weight_tensor_td_conv_new = saving.preprocess_weights_for_loading(
         td_conv_layer,
         weight_tensor_td_conv_old,
         original_keras_version='1')
@@ -554,34 +529,35 @@ def test_load_layers():
     weight_value_tuples += zip(symbolic_weights, weight_tensor_td_conv_new)
 
     # Bidirectional ConvLSTM2D layer
-    # old ConvLSTM2D took a list of 12 weight tensors, returns a list of 3 concatenated larger tensors.
-    weight_tensor_bi_convlstm_old = []
+    # old ConvLSTM2D took a list of 12 weight tensors,
+    # returns a list of 3 concatenated larger tensors.
+    weights_bi_conv_old = []
     for j in range(2):  # bidirectional
         for i in range(4):
-            weight_tensor_bi_convlstm_old.append(np.zeros((3, 3, 15, 10)))  # kernel
-            weight_tensor_bi_convlstm_old.append(np.zeros((3, 3, 10, 10)))  # recurrent kernel
-            weight_tensor_bi_convlstm_old.append(np.zeros((10,)))  # bias
+            weights_bi_conv_old.append(np.zeros((3, 3, 15, 10)))  # kernel
+            weights_bi_conv_old.append(np.zeros((3, 3, 10, 10)))  # recurrent kernel
+            weights_bi_conv_old.append(np.zeros((10,)))  # bias
 
     bi_convlstm_layer = model.layers[2]
-    weight_tensor_bi_convlstm_new = topology.preprocess_weights_for_loading(
+    weights_bi_conv_new = saving.preprocess_weights_for_loading(
         bi_convlstm_layer,
-        weight_tensor_bi_convlstm_old,
+        weights_bi_conv_old,
         original_keras_version='1')
 
     symbolic_weights = bi_convlstm_layer.weights
-    assert (len(symbolic_weights) == len(weight_tensor_bi_convlstm_new))
-    weight_value_tuples += zip(symbolic_weights, weight_tensor_bi_convlstm_new)
+    assert (len(symbolic_weights) == len(weights_bi_conv_new))
+    weight_value_tuples += zip(symbolic_weights, weights_bi_conv_new)
 
     K.batch_set_value(weight_value_tuples)
 
     assert np.all(K.eval(model.layers[1].weights[0]) == weight_tensor_td_conv_new[0])
     assert np.all(K.eval(model.layers[1].weights[1]) == weight_tensor_td_conv_new[1])
-    assert np.all(K.eval(model.layers[2].weights[0]) == weight_tensor_bi_convlstm_new[0])
-    assert np.all(K.eval(model.layers[2].weights[1]) == weight_tensor_bi_convlstm_new[1])
-    assert np.all(K.eval(model.layers[2].weights[2]) == weight_tensor_bi_convlstm_new[2])
-    assert np.all(K.eval(model.layers[2].weights[3]) == weight_tensor_bi_convlstm_new[3])
-    assert np.all(K.eval(model.layers[2].weights[4]) == weight_tensor_bi_convlstm_new[4])
-    assert np.all(K.eval(model.layers[2].weights[5]) == weight_tensor_bi_convlstm_new[5])
+    assert np.all(K.eval(model.layers[2].weights[0]) == weights_bi_conv_new[0])
+    assert np.all(K.eval(model.layers[2].weights[1]) == weights_bi_conv_new[1])
+    assert np.all(K.eval(model.layers[2].weights[2]) == weights_bi_conv_new[2])
+    assert np.all(K.eval(model.layers[2].weights[3]) == weights_bi_conv_new[3])
+    assert np.all(K.eval(model.layers[2].weights[4]) == weights_bi_conv_new[4])
+    assert np.all(K.eval(model.layers[2].weights[5]) == weights_bi_conv_new[5])
 
 
 def convert_weights(layer, weights):
@@ -598,42 +574,68 @@ def convert_weights(layer, weights):
     return weights
 
 
-@keras_test
 @pytest.mark.parametrize("layer", [
     layers.GRU(2, input_shape=[3, 5]),
     layers.LSTM(2, input_shape=[3, 5]),
     layers.ConvLSTM2D(5, (3, 3),
                       input_shape=[6, 6, 6, 6],
                       data_format='channels_first'),
-])
+], ids=['GRU', 'LSTM', 'ConvLSTM2D'])
 def test_preprocess_weights_for_loading(layer):
-    model = Sequential([layer])
+    # A model is needed to initialize weights.
+    _ = Sequential([layer])
     weights1 = layer.get_weights()
-    weights2 = topology.preprocess_weights_for_loading(
+    weights2 = saving.preprocess_weights_for_loading(
         layer, convert_weights(layer, weights1),
         original_keras_version='1')
     assert all([np.allclose(x, y, 1e-5)
                 for (x, y) in zip(weights1, weights2)])
 
 
-@keras_test
 @pytest.mark.parametrize("layer", [
     layers.Conv2D(2, (3, 3), input_shape=[5, 5, 3]),
     layers.Conv2DTranspose(2, (5, 5),
                            input_shape=[7, 7, 3],
                            data_format='channels_first'),
-])
+], ids=['Conv2D', 'Conv2DTranspose'])
 def test_preprocess_weights_for_loading_for_model(layer):
     model = Sequential([layer])
     weights1 = model.get_weights()
-    weights2 = topology.preprocess_weights_for_loading(
+    weights2 = saving.preprocess_weights_for_loading(
         model, convert_weights(layer, weights1),
         original_keras_version='1')
     assert all([np.allclose(x, y, 1e-5)
                 for (x, y) in zip(weights1, weights2)])
 
 
-@keras_test
+@pytest.mark.parametrize('layer_class,args', [
+    (layers.GRU, {'units': 2, 'input_shape': [3, 5]}),
+    (layers.GRU, {'units': 2, 'input_shape': [3, 5], 'reset_after': True}),
+    (layers.LSTM, {'units': 2, 'input_shape': [3, 5]}),
+])
+def test_preprocess_weights_for_loading_rnn_should_be_idempotent(layer_class, args):
+    """
+    Loading weights from a RNN class to itself should not convert the weights.
+    """
+    # layer can be instantiated only for supported backends
+    layer = layer_class(**args)
+    # A model is needed to initialize weights.
+    _ = Sequential([layer])
+    weights1 = layer.get_weights()
+    weights2 = saving.preprocess_weights_for_loading(layer, weights1)
+    assert all([np.allclose(x, y, 1e-5) for (x, y) in zip(weights1, weights2)])
+
+
+@pytest.mark.parametrize('layer_class,args', [
+    (layers.CuDNNGRU, {'units': 2, 'input_shape': [3, 5]}),
+    (layers.CuDNNLSTM, {'units': 2, 'input_shape': [3, 5]}),
+])
+@skipif_no_tf_gpu
+def test_preprocess_weights_for_loading_cudnn_rnn_should_be_idempotent(layer_class,
+                                                                       args):
+    test_preprocess_weights_for_loading_rnn_should_be_idempotent(layer_class, args)
+
+
 def test_recursion_with_bn_and_loss():
     model1 = Sequential([
         layers.Dense(5, input_dim=5, activity_regularizer='l1'),
@@ -660,12 +662,34 @@ def test_recursion_with_bn_and_loss():
     model2.fit(x, y, verbose=0, epochs=1)
 
 
-@keras_test
+def test_activity_regularization_with_model_composition():
+
+    def reg(x):
+        return K.sum(x)
+
+    net_a_input = Input((2,))
+    net_a = net_a_input
+    net_a = Dense(2, kernel_initializer='ones',
+                  use_bias=False,
+                  activity_regularizer=reg)(net_a)
+    model_a = Model([net_a_input], [net_a])
+
+    net_b_input = Input((2,))
+    net_b = model_a(net_b_input)
+    model_b = Model([net_b_input], [net_b])
+
+    model_b.compile(optimizer='sgd', loss=None)
+    x = np.ones((1, 2))
+    loss = model_b.evaluate(x)
+    assert loss == 4
+
+
 def test_shared_layer_depth_is_correct():
-    # Basic outline here: we have a shared embedding layer, and two inputs that go through
-    # different depths of computation in the graph before the final output.  We need the computed
-    # depth of the input layers to be the same, because they both pass through the embedding layer
-    # before anything else happens.  That's what we're testing.
+    # Basic outline here: we have a shared embedding layer, and two inputs that
+    # go through different depths of computation in the graph before
+    # the final output.  We need the computed depth of the input layers to be
+    # the same, because they both pass through the embedding layer before anything
+    # else happens.  That's what we're testing.
     from keras.layers import Embedding, Input, Dense, Concatenate
     from keras.models import Model
     input1 = Input(shape=(10,), name='input1')
@@ -678,7 +702,7 @@ def test_shared_layer_depth_is_correct():
     model = Model(inputs=[input1, input2], outputs=final_output)
     input1_depth = -1
     input2_depth = -1
-    for depth, layers in model.layers_by_depth.items():
+    for depth, layers in model._layers_by_depth.items():
         for layer in layers:
             if layer.name == 'input1':
                 input1_depth = depth
@@ -688,7 +712,6 @@ def test_shared_layer_depth_is_correct():
     assert input1_depth == input2_depth
 
 
-@keras_test
 def test_layer_sharing_at_heterogeneous_depth():
     x_val = np.random.random((10, 5))
 
@@ -710,7 +733,6 @@ def test_layer_sharing_at_heterogeneous_depth():
     np.testing.assert_allclose(output_val, output_val_2, atol=1e-6)
 
 
-@keras_test
 def test_layer_sharing_at_heterogeneous_depth_with_concat():
     input_shape = (16, 9, 3)
     input_layer = Input(shape=input_shape)
@@ -738,34 +760,48 @@ def test_layer_sharing_at_heterogeneous_depth_with_concat():
     np.testing.assert_allclose(output_val, output_val_2, atol=1e-6)
 
 
-@keras_test
 def test_multi_output_mask():
     """Fixes #7589"""
-    class ArbitraryMultiOutputLayer(Layer):
+    class TestMultiOutputLayer(Layer):
         def __init__(self, **kwargs):
-            super(ArbitraryMultiOutputLayer, self).__init__(**kwargs)
+            super(TestMultiOutputLayer, self).__init__(**kwargs)
 
         def call(self, inputs, **kwargs):
             return [K.abs(inputs), K.abs(inputs)]
 
         def compute_output_shape(self, input_shape):
-            out_shape = super(ArbitraryMultiOutputLayer, self).compute_output_shape(input_shape)
+            out_shape = super(TestMultiOutputLayer, self).compute_output_shape(
+                input_shape)
             return [out_shape, out_shape]
 
-    class ArbitraryMultiInputLayer(Layer):
+    class TestMultiInputLayer(Layer):
         def __init__(self, **kwargs):
-            super(ArbitraryMultiInputLayer, self).__init__(**kwargs)
+            super(TestMultiInputLayer, self).__init__(**kwargs)
 
         def call(self, inputs, **kwargs):
             negative, positive = inputs
             return negative + positive
 
     input_layer = Input(shape=(16, 16, 3))
-    x, y = ArbitraryMultiOutputLayer()(input_layer)
-    z = ArbitraryMultiInputLayer()([x, y])
+    x, y = TestMultiOutputLayer()(input_layer)
+    z = TestMultiInputLayer()([x, y])
     _ = Model(inputs=input_layer, outputs=z)
     assert K.int_shape(z)[1:] == (16, 16, 3)
 
 
+def test_constant_initializer_with_numpy():
+    model = Sequential()
+    model.add(Dense(2, input_shape=(3,),
+                    kernel_initializer=Constant(np.ones((3, 2)))))
+    model.add(Dense(3))
+    model.compile(loss='mse', optimizer='sgd', metrics=['acc'])
+
+    json_str = model.to_json()
+    model_from_json(json_str).summary()
+
+    yaml_str = model.to_yaml()
+    model_from_yaml(yaml_str).summary()
+
+
 if __name__ == '__main__':
     pytest.main([__file__])
